--[[ **************************************************************************
     * Add On Starts Here                                                     *
	 **************************************************************************
]]

local on = true
local off = false

CAGI_Defaults = {
-- Default Settings   All settings can be changed in game with
-- the /cagi command (command = variable name (case insensitive))
Enable = on, -- enable/disable the addon 
Debug = off,  -- enable debug messages
Serious = off,	-- Strict / serious messages only
Solo = on, -- Enable announces to yourself
Opp = on, -- Enable announcing Opposing Sides stuff in BG
RaidWarning = on, --Enable announcing to Raid Warnings if you can
Mammoths = on, --Enable announcing of Mammoths
Links = on, --Display detected events as a link
}

CAGI_Settings = {}

CAGI={}

CAGI.Frame = CreateFrame("Frame")
CAGI.Frame:RegisterEvent("ADDON_LOADED")
CAGI.Frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
CAGI.Frame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");

-- Local text colors
local yel = "|cFFFFFF00"
local wht = "|cFFFFFFFF"
local blu = "|cFF00FFFF"
local red = "|cFFFF0000"
local grn = "|cFF00FF00"
local res = "|r"

-- General settings

-- Bill's Utils
local addOptionMt = BillsUtils.addOptionMt
local StatColor = BillsUtils.StatColor
local SJprint = BillsUtils.SJprint
local OptSaveTF = BillsUtils.OptSaveTF
local IsPvPZone = BillsUtils.IsPvPZone
local GenderWord = BillsUtils.GenderWord

BillsUtils.Locals[#BillsUtils.Locals +1] = function () --function is called if/when a newer version of Bill's Utils loads after the addon is loaded
	addOptionMt = BillsUtils.addOptionMt
	StatColor = BillsUtils.StatColor
	SJprint = BillsUtils.SJprint
	OptSaveTF = BillsUtils.OptSaveTF
	IsPvPZone = BillsUtils.IsPvPZone
	GenderWord = BillsUtils.GenderWord
end


local LastCast = 0
local LastBy = ""
local LastTime = 0
CAGI.Frame:SetScript("OnEvent",function(self, event,...)
	if event == "ADDON_LOADED" then
		local arg = ...
		if arg ~= "ComeAndGetIt" then
			return
		end
	
		SJprint("\n", grn, "Come And Get It loaded\n", res)
		addOptionMt( CAGI_Settings , CAGI_Defaults )
		return self, event, ...
	end
	
	if not(CAGI_Settings.Enable) then
		return
	end
	
	local BG, RW, Raid, Arena, Party = false, false, false, false, false
	local mode = "none"
	
	local _, IType = IsInInstance()
	-- select alert type (only 1 should be true at any time)
	if IType == "pvp" then
		BG = true
		mode = "BATTLEGROUND"
	elseif IType == "arena" then
		Arena = true
		mode = "PARTY"
	elseif GetNumRaidMembers() > 0 then
		if CAGI_Settings.RaidWarning and (IsRaidLeader("player") == 1) or (IsRaidOfficer("player") == 1) then
			RW = true
			mode = "RAID_WARNING"
		else
			Raid = true
			mode = "RAID"
		end
	elseif GetNumPartyMembers() > 0 then
		Party = true
		mode = "PARTY"
	end
	
	if CAGI_Settings.Debug then
		print("CAGI Debug: Party, Raid, BG, RW, IType")
		print(Party, Raid, BG, RW, IType)
	end

	if not( BG or Arena or Raid or RW or Party ) and not(CAGI_Settings.Debug) and not(CAGI_Settings.Solo) then
		return
	end

	local match = false
	local OutputMsg = ""
	local etype, sourceGUID, sourceName, destName, spellId, spellName
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		_, etype, _, sourceGUID, sourceName, _, _, _, destName, _, _, spellId, spellName = ...
		if etype == nil then
			etype = "nil"
		end
		if CAGI_Settings.Debug then
			print("CAGI Debug: event = "..event.."  etype = "..etype.."  sourceName = "..sourceName.."   spellId = "..spellId.."   spellName = "..spellName )
		end
		
		if etype == "SPELL_CAST_SUCCESS" then
			
			-- ****************************************************************
			-- * ENGINEERING BOTS GO HERE                                     *
			-- ****************************************************************
			if not(match) then
				local SpellList = {54711, 54710, 67826, 22700, 44389 } 
				--  "Scrapbot", "MOLL-E", "Jeeves", "Field Repair Bot 74A", "Field Repair Bot 110G", 
				for x = 1, #SpellList do
					if spellId == SpellList[x] then
						match = true
						local PrefixTable = {"built a", "assembled together a", "hammered together a", "assembled "..GenderWord(unit, "himself", sourceGUID ).." a" }
						local SuffixTable = {"Beware of sharp edges.", "A hammer and duct tape can be amazing.", "I think there is a screw loose.", "I got hit by a cog!", "Why are there so many extra parts?"}
						OutputMsg = "%s "..PrefixTable[math.random(1, #PrefixTable)].." %s. "..SuffixTable[math.random(1, #SuffixTable)]
						break
					end
				end
			end
			
			-- ****************************************************************
			-- * RITUALS GO HERE                                              *
			-- ****************************************************************
			if not(match) then
				local SpellList = {698, 29893, 43987}
				--  "Ritual of Summoning", "Ritual of Souls", "Ritual of Refreshment"
				for x = 1, #SpellList do
					if spellId == SpellList[x] then
						match = true
						local PrefixTable = {"is making a portal for a", "is trying to tear a hole in space for a", "is casting the fabled" }
						local SuffixTable = {"Clickey Clickey!", "Lend a hand or paw.", "Just Click it.", "Help "..GenderWord(unit, "him",sourceGUID ).." summon it up."}
						OutputMsg = "%s "..PrefixTable[math.random(1, #PrefixTable)].." %s. "..SuffixTable[math.random(1, #SuffixTable)]
						break
					end
				end
			end
			
		elseif etype == "SPELL_CREATE" then
			
			-- ****************************************************************
			-- * FOOD ITEMS GO HERE                                           *
			-- ****************************************************************
			local SpellList = {58465, 58474, 87643, 87644, 57426,
				57301, 66476, 87915}
			--	"Gigantic Feast", "Small Feast", "Broiled Dragon Feast", "Seafood Magnifique Feast", "Fish Feast",
			--	"Great Feast", "Bountiful Feast", "Goblin Barbeque" 
			
			for x = 1, #SpellList do
				if spellId == SpellList[x] then
					match = true
					local PrefixTable = {"cooked up a nice" , "made a nice steaming", "made a big spread of", "made a tastey", "may have burned the", "made "..GenderWord(sourceName, "his", sourceGUID).." special"}
					local SuffixTable = {"Dig in!", "Chow down!", "Thanks for the grub.", "Free Food!", "It tastes like chicken!", "Thank "..GenderWord(sourceName, "him", sourceGUID).." for the food!"}
					OutputMsg = "%s "..PrefixTable[math.random(1, #PrefixTable)].." %s. "..SuffixTable[math.random(1, #SuffixTable)]
					break
				end
			end
			
			-- ****************************************************************
			-- * CAULDRONS GO HERE                                            *
			-- ****************************************************************
			if not(match) then
				local SpellList = {92654, 92692, 92652, 92693, -- "Big Cauldron of Battle"
						92635, 92689, 92645, 92691, -- "Cauldron of Battle"
						41443, -- "Cauldron of Major Arcane Protection"
						41494, --  "Cauldron of Major Fire Protection"
						41495, -- "Cauldron of Major Frost Protection"
						41497, -- "Cauldron of Major Nature Protection"
						41498} -- "Cauldron of Major Shadow Protection"
				for x = 1, #SpellList do
					if spellId == SpellList[x] then
						match = true
						local PrefixTable = {"brewed up a", "laid out a bubbling", "mixed up a", "brewed a bubbling"}
						local SuffixTable = {"Better living through chemistry.", "The bubbles tickle my nose.", "It may taste bad, but it's good for you.", "This is not your mother's medicine"}
						OutputMsg = "%s "..PrefixTable[math.random(1, #PrefixTable)].." %s. "..SuffixTable[math.random(1, #SuffixTable)]
						break
					end
				end
			end
		elseif etype == "SPELL_RESURRECT" then

			-- ****************************************************************
			-- * REBIRTH GOES HERE                                            *
			-- ****************************************************************
			if spellId == 20484 then  
				match = true
				OutputMsg = "%s casts %s on "..destName
			end
			
			-- ****************************************************************
			-- * REINCARNATION GOES HERE                                      *
			-- ****************************************************************
			if not(match) then
				if spellId == 20608 then
					match = true
					OutputMsg = "%s casts %s on "..GenderWord(sourceName, "himself", sourceGUID )
				end
			end
			

		end

	elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
	-- no GUID returned so no need to pass to GenderWord
		local unit = ""
		unit, spellName, _, _, spellId = ...
		sourceName = GetUnitName(unit, true)
		
		if CAGI_Settings.Debug then
			print("CAGI Debug: event = "..event.."   sourceName = "..sourceName.."   spellId = "..spellId.."   spellName = "..spellName )
		end
		
		-- ****************************************************************
		-- * Traveler's Tundra Mammoth                                    *
		-- ****************************************************************
		local SpellList = {61447, 61425}
		--"Traveler's Tundra Mammoth-Horde", "Traveler's Tundra Mammoth-Alliance"
		for x = 1, #SpellList do
			if spellId == SpellList[x] then
				if not CAGI_Settings.Mammoths then
					return
				end
				match = true
				local PrefixTable = {"is riding on", "is mounted on", "climbed up on", "mounted", "summoned" }
				local SuffixTable = {"Hi ho Silver, Away!", "For your safety, stand to the front of the mount.", "Toxic exhaust!", "Vendors now open for business.", "Bet "..GenderWord( unit, "he").." wishes "..GenderWord(unit, "he").." got a cut from the vendors."}
				OutputMsg = "%s "..PrefixTable[math.random(1, #PrefixTable)].." "..GenderWord(unit, "his").." %s. "..SuffixTable[math.random(1, #SuffixTable)]
				break
			end
		end

		-- ****************************************************************
		-- * Portals go here                                              *
		-- ****************************************************************
		if not(match) then
			local SpellList = {11419, 32266, 11416, 10059, 49360,
				11417, 32267, 11420,  11418, 49361,
				88345, 99346}
			-- "Portal: Darnassus", "Portal: Exodar", "Portal: Ironforge", "Portal: Stormwind", "Portal: Theramore",
			-- "Portal: Orgrimmar", "Portal: Silvermoon", "Portal: Thunder Bluff", "Portal: Undercity", "Portal: Stonard"
			-- "Portal: Tol Barad - Alliance", "Portal: Tol Barad - Horde"
			for x = 1, #SpellList do
				if spellId == SpellList[x] then
					match = true
					local PrefixTable = {"is casting", "is ripping a hole in space for a" }
					local SuffixTable = {"Make your escape!", "See new and exciting places.", "Have mage, Will Travel.", "Let's thank "..GenderWord(unit, "him") }
					OutputMsg = "%s "..PrefixTable[math.random(1, #PrefixTable)].." %s. "..SuffixTable[math.random(1, #SuffixTable)]
					break
				end
			end
		end
		
		
	else
		return
	end
	
	if not(match) then
		return
	end
	
	if time() - LastTime < 3 and LastSpell == spellId and LastBy == sourceName then
		if CAGI_Settings.Debug then
			print("CAGI Debug: An echo event just happened")
		end
		return
	end
	
	local name = sourceName
	if string.find(name," ") ~= nil then
		name = string.split(" ", name)
	end
	if string.find(name,"-") ~= nil then
		name = string.split("-", name)
	end
	
	local InGroup = UnitInParty(sourceName) ~= nil or UnitInRaid(sourceName) ~= nil or UnitInBattleground(sourceName) ~= nil or UnitIsUnit("player", sourceName) ~= nil
	InGroup = InGroup or UnitInParty(name) ~= nil or UnitInRaid(name) ~= nil or UnitInBattleground(name) ~= nil or UnitIsUnit("player", name) ~= nil
	
	if not(InGroup) and not(CAGI_Settings.Debug) and not(CAGI_Settings.Solo) and not(CAGI_Settings.Opp) then
		return
	end
	
	if CAGI_Settings.Links then
		spellName = "|cff71d5ff|Hspell:"..spellId.."|h["..spellName.."]|h|r"
	end
	
	local msg = " "
	if CAGI_Settings.Serious then
		msg = string.format("%s made a %s. " ,name, spellName)
	else		
		msg = string.format(OutputMsg, name, spellName )
	end
	
	if not(InGroup) and BG and CAGI_Settings.Opp then
		if spellId == 20484 then -- Rebirth
			msg = string.format("Opposing Side: %s casts %s on %s", name, spellName, destName)
		elseif spellId == 61447 or spellId == 61425 then -- Mammoths
			return
		else
			msg = string.format("Opposing Side: %s makes/creates/casts %s ", name, spellName)
		end
	end
	
	if mode ~= "none" then
		SendChatMessage( "CAGI: "..msg, mode )
	end
	
	if CAGI_Settings.Debug or CAGI_Settings.Solo then
		print(string.format("CAGI: "..msg ))
	end

	LastTime = time()
	LastSpell = spellId
	LastBy = sourceName
	return
end)
	

SLASH_CAGI1 = "/cagi"
SlashCmdList["CAGI"] = function(msg)
	local cmd,arg = string.split(" ", msg)
	cmd = cmd:lower()
	
	if cmd =="config" then
		InterfaceOptionsFrame_OpenToCategory( CAGI.panel )
		return
	end
	
	if not(CAGI_Settings.Enable) and cmd ~= "enable" then
		print(" ")
		SJprint( yel, "Come And Get It", res)
		SJprint( yel, "  This addon is ", red, "DISABLED", res)
		SJprint( yel, "  Type: ", blu, "/cagi enable on", res)
		SJprint( yel, "  to enable.", res)
		return
	end

	local Command = {"Enable", "Debug", "Serious", "Solo", "Opp", "RaidWarning", "Mammoths", "Links" }
	
	for x = 1, #Command do
		if cmd == string.lower(Command[x]) then
			OptSaveTF( CAGI_Settings, Command[x], arg)
			SJprint( yel, "Come And Get It:", Command[x],"is", StatColor( CAGI_Settings[Command[x]] ), res)
			return
		end
	end

	if cmd == "" then
		print(" ")
		SJprint( yel, "________________________________________", res)
		SJprint( yel, "Come And Get It", res)
		SJprint( yel, "  Addon State:", res)
		for x = 1, #Command do
			SJprint( yel, Command[x], wht, "is", StatColor( CAGI_Settings[Command[x]] ), res)
		end
		SJprint( yel, "Available commands are listed above in", blu, "BLUE.", res)
		SJprint( yel, "Type", blu, "/cagi config", yel, "to open the GUI config screen.", res)
		SJprint( yel, "Type", blu, "/cagi cmdlist", yel, "for a description of commands.", res)
		SJprint( yel, "________________________________________", res)
		return
	end

	if cmd == "cmdlist" or cmd == "help" then
		print(" ")
		SJprint( yel, "________________________________________", res)
		SJprint( yel, "Come And Get It", res)
		SJprint( yel, "  Addon State:", res)
		SJprint( blu, "     Enable", wht, "Turns the Add On on and off", res)
		SJprint( blu, "     Debug", wht, "Turns debug messages on and off", res)
		SJprint( blu, "     Serious", wht, "Turns Serious/Strict messages on and off", res)
		SJprint( blu, "     Solo", wht, "Enable announcements to yourself on and off", res)
		SJprint( blu, "     Opp", wht, "Enable opposing side announcements on and off", res)
		SJprint( blu, "     RaidWarning", wht, "Enable announcements in RaidWarnings if you can use them", res)
		SJprint( blu, "     Mammoths", wht, "Enable announcements when someone mounts a Traveler's Tundra Mammoth", res)
		SJprint( blu, "     Links", wht, "Output detected items as a link instead of just text", res)
		SJprint( yel, "Available commands are listed above in ", blu, "BLUE.", res)
		SJprint( yel, "Type ", blu, "/cagi config",  yel, " to open the GUI config screen.", res)
		SJprint( yel, "________________________________________", res)
		return
	end

	SJprint( yel, "  Come And Get It", res)
	SJprint( yel, "  Error: ", red, cmd, yel, " is not a valid command", res)
	SJprint( yel, "  Type: ", blu, "/cagi", yel, "for the addon status and command list", res)
	SJprint( yel, "  Type: ", blu, "/cagi cmdlist", yel, "for command list help", res)
	return
end
